/* ECE 4760 Final Project: Virtual Saxophone
   Chris Piccoli (ctp33)
   Suryansh Agarwal (sa488)
   May 4, 2012 */
   
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <math.h> 
#include <stdio.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include "uart.h"


// set up serial for debugging
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

// timeout values for each task
#define t1 200  
#define t2 30
#define t3 500

// Sound synthesis constants
#define max_amp 32767

// Frequency multiply constants
#define SLOW 8.192

// Begin and end definitions
#define begin {
#define end }

// Frequency assigned to each note
// Range from AS2 to F5
#define AS2		116.540
#define B2		123.470
#define C3		130.810
#define CS3		138.590
#define D3		146.830
#define DS3		155.560
#define E3		164.810
#define F3		174.610
#define FS3		185.000
#define G3		196.000
#define GS3		207.650
#define A3		220.000
#define AS3		233.080
#define B3		246.940
#define C4		261.630
#define CS4		277.180
#define D4		293.660
#define DS4		311.130
#define E4		329.630
#define F4		349.230
#define FS4		369.990
#define G4		392.000
#define GS4		415.300
#define A4		440.000
#define AS4		466.160
#define B4		493.880
#define C5		523.250
#define CS5		554.370
#define D5		587.330
#define DS5		622.250
#define E5		659.260
#define F5		698.460

// The DDS variables 
volatile unsigned int acc_1, acc_2, acc_3, acc_4, acc_5, acc_6, acc_7, acc_8, acc_9, acc_10;
volatile unsigned int inc_1, inc_2, inc_3, inc_4, inc_5, inc_6, inc_7, inc_8, inc_9, inc_10 ;
volatile unsigned char high_1, high_2, high_3, high_4, high_5, high_6, high_7, high_8, high_9, high_10 ;
 
// attack and decay constants
volatile unsigned char rise_1, rise_2, rise_3, rise_4, rise_5, rise_6, rise_7, rise_8, rise_9, rise_10 ;
volatile unsigned char fall_1, fall_2, fall_3, fall_4, fall_5, fall_6, fall_7, fall_8, fall_9, fall_10 ; 
// attack and decay waveforms
volatile int amp_rise_1, amp_rise_2 ,amp_rise_3, amp_rise_4, amp_rise_5 ,amp_rise_6, amp_rise_7, amp_rise_8 ,amp_rise_9, amp_rise_10 ;
volatile int amp_fall_1, amp_fall_2 ,amp_fall_3, amp_fall_4, amp_fall_5 ,amp_fall_6, amp_fall_7, amp_fall_8 ,amp_fall_9, amp_fall_10;

// amplitude waveforms = attack * decay waveforms
volatile int amp_1, amp_2 ,amp_3, amp_4, amp_5, amp_6, amp_7, amp_8, amp_9, amp_10;

// amplitude scaling constant
volatile unsigned char a_1, a_2, a_3, a_4, a_5, a_6, a_7, a_8, a_9, a_10 ; 

// tables for DDS and the output waves to be added together			
signed char sineTable[256], sine_1, sine_2, sine_3, sine_4, sine_5, sine_6, sine_7, sine_8, sine_9, sine_10 ;

// Trigger
volatile char pluck, pushed ;

// Used to determine number of harmonics
volatile char num_harmonics;

// Time variables
// the volitile is needed because the time is only set in the ISR
// time counts mSec, sample counts DDS samples (62.5 KHz)
volatile unsigned int time ;
volatile char  count;

// index for sine table build
volatile unsigned int i;

// Frequency of music scale
volatile float freq;

// Variable used for probing ISR
volatile unsigned int probe, ok_read;

// Indicates if key is pressed
unsigned char key1_on, key2_on, key3_on, key4_on, key5_on, key6_on, key7_on, key8_on,
key9_on, key10_on, key11_on, key12_on, key13_on, key14_on, key15_on, key16_on, key17_on,
key18_on, key19_on, key20_on;

unsigned char portd_keys, portc_keys, porta_keys;

char Ain ; 		//raw A to D number
char play_note;	
char vol;		
char count_val;	  

// Task subroutines
void check_fingering(void);			// Adjust note based on fingering

// Initialize subroutine
void initialize(void); 				// all the usual mcu stuff 

volatile unsigned int time1, time2, time3;			// Timer for LCD task and keypad task


//**********************************************************
//timer 1 compare ISR
ISR (TIMER1_COMPA_vect) 
begin 
	ok_read = 0;
	probe = TCNT1;
	

	// compute exponential attack and decay of amplitude
	// the (time & 0x0ff) slows down the decay computation by 256 times		
	if (play_note == 0)
	begin
	if ((time & 0x0ff) == 0) begin
		amp_fall_1 = amp_fall_1 - (amp_fall_1>>fall_1) ;
		amp_rise_1 = amp_rise_1 - (amp_rise_1>>rise_1);
		amp_fall_2 = amp_fall_2 - (amp_fall_2>>fall_2) ;
		amp_rise_2 = amp_rise_2 - (amp_rise_2>>rise_2);
		amp_fall_3 = amp_fall_3 - (amp_fall_3>>fall_3) ;
		amp_rise_3 = amp_rise_3 - (amp_rise_3>>rise_3);
		
		amp_fall_4 = amp_fall_4 - (amp_fall_4>>fall_4) ;
		amp_rise_4 = amp_rise_4 - (amp_rise_4>>rise_4);
		amp_fall_5 = amp_fall_5 - (amp_fall_5>>fall_5) ;
		amp_rise_5 = amp_rise_5 - (amp_rise_5>>rise_5);
		amp_fall_6 = amp_fall_6 - (amp_fall_6>>fall_6) ;
		amp_rise_6 = amp_rise_6 - (amp_rise_6>>rise_6);
		amp_fall_7 = amp_fall_7 - (amp_fall_7>>fall_7) ;
		amp_rise_7 = amp_rise_7 - (amp_rise_7>>rise_7);
		amp_fall_8 = amp_fall_8 - (amp_fall_8>>fall_8) ;
		amp_rise_8 = amp_rise_8 - (amp_rise_8>>rise_8);
		amp_fall_9 = amp_fall_9 - (amp_fall_9>>fall_9) ;
		amp_rise_9 = amp_rise_9 - (amp_rise_9>>rise_9);
		amp_fall_10 = amp_fall_10 - (amp_fall_10>>fall_10) ;
		amp_rise_10 = amp_rise_10 - (amp_rise_10>>rise_10);
	end

	// form (1-exp(-t/tau)) for the attack phase
	// product of rise and fall exponentials is the amplitude envelope
	amp_1 = ((max_amp - amp_rise_1)>>8) * (amp_fall_1>>8) ;
	amp_2 = ((max_amp - amp_rise_2)>>8) * (amp_fall_2>>8) ;
	amp_3 = ((max_amp - amp_rise_3)>>8) * (amp_fall_3>>8) ;
	
	amp_4 = ((max_amp - amp_rise_4)>>8) * (amp_fall_4>>8) ;
	amp_5 = ((max_amp - amp_rise_5)>>8) * (amp_fall_5>>8) ;
	amp_6 = ((max_amp - amp_rise_6)>>8) * (amp_fall_6>>8) ;
	amp_7 = ((max_amp - amp_rise_7)>>8) * (amp_fall_7>>8) ;
	amp_8 = ((max_amp - amp_rise_8)>>8) * (amp_fall_8>>8) ;
	amp_9 = ((max_amp - amp_rise_9)>>8) * (amp_fall_9>>8) ;
	amp_10 = ((max_amp - amp_rise_10)>>8) * (amp_fall_10>>8) ;
	end

	// Init the synth
	if (pluck==1) begin
		amp_fall_1 = max_amp; 
		amp_rise_1 = max_amp ;
		amp_fall_2 = max_amp; 
		amp_rise_2 = max_amp ;
		amp_fall_3 = max_amp; 
		amp_rise_3 = max_amp ;	

		amp_fall_4 = max_amp; 
		amp_rise_4 = max_amp ;
		amp_fall_5 = max_amp; 
		amp_rise_5 = max_amp ;
		amp_fall_6 = max_amp; 
		amp_rise_6 = max_amp ;
		amp_fall_7 = max_amp; 
		amp_rise_7 = max_amp ;
		amp_fall_8 = max_amp; 
		amp_rise_8 = max_amp ;
		amp_fall_9 = max_amp; 
		amp_rise_9 = max_amp ;
		amp_fall_10 = max_amp; 
		amp_rise_10 = max_amp ;
		
		// phase lock the synth
		acc_1 = 0 ;
		acc_2 = 0 ;
		acc_3 = 0 ;
		
		acc_4 = 0 ;
		acc_5 = 0 ;
		acc_6 = 0 ;
		acc_7 = 0 ;
		acc_8 = 0 ;
		acc_9 = 0 ;
		acc_10 = 0 ;
		
		pluck = 0;
		
		amp_fall_1 = amp_fall_1 - (amp_fall_1>>fall_1) ;
		amp_rise_1 = amp_rise_1 - (amp_rise_1>>rise_1);
		amp_fall_2 = amp_fall_2 - (amp_fall_2>>fall_2) ;
		amp_rise_2 = amp_rise_2 - (amp_rise_2>>rise_2);
		amp_fall_3 = amp_fall_3 - (amp_fall_3>>fall_3) ;
		amp_rise_3 = amp_rise_3 - (amp_rise_3>>rise_3);
		
		amp_fall_4 = amp_fall_4 - (amp_fall_4>>fall_4) ;
		amp_rise_4 = amp_rise_4 - (amp_rise_4>>rise_4);
		amp_fall_5 = amp_fall_5 - (amp_fall_5>>fall_5) ;
		amp_rise_5 = amp_rise_5 - (amp_rise_5>>rise_5);
		amp_fall_6 = amp_fall_6 - (amp_fall_6>>fall_6) ;
		amp_rise_6 = amp_rise_6 - (amp_rise_6>>rise_6);
		amp_fall_7 = amp_fall_7 - (amp_fall_7>>fall_7) ;
		amp_rise_7 = amp_rise_7 - (amp_rise_7>>rise_7);
		amp_fall_8 = amp_fall_8 - (amp_fall_8>>fall_8) ;
		amp_rise_8 = amp_rise_8 - (amp_rise_8>>rise_8);
		amp_fall_9 = amp_fall_9 - (amp_fall_9>>fall_9) ;
		amp_rise_9 = amp_rise_9 - (amp_rise_9>>rise_9);
		amp_fall_10 = amp_fall_10 - (amp_fall_10>>fall_10) ;
		amp_rise_10 = amp_rise_10 - (amp_rise_10>>rise_10);
		
		amp_1 = ((max_amp - amp_rise_1)>>8) * (amp_fall_1>>8) ;
		amp_2 = ((max_amp - amp_rise_2)>>8) * (amp_fall_2>>8) ;
		amp_3 = ((max_amp - amp_rise_3)>>8) * (amp_fall_3>>8) ;
		
		amp_4 = ((max_amp - amp_rise_4)>>8) * (amp_fall_4>>8) ;
		amp_5 = ((max_amp - amp_rise_5)>>8) * (amp_fall_5>>8) ;
		amp_6 = ((max_amp - amp_rise_6)>>8) * (amp_fall_6>>8) ;
		amp_7 = ((max_amp - amp_rise_7)>>8) * (amp_fall_7>>8) ;
		amp_8 = ((max_amp - amp_rise_8)>>8) * (amp_fall_8>>8) ;
		amp_9 = ((max_amp - amp_rise_9)>>8) * (amp_fall_9>>8) ;
		amp_10 = ((max_amp - amp_rise_10)>>8) * (amp_fall_10>>8) ;
	end

	//the DDRs -- 
	acc_1 = acc_1 + inc_1 ;
	high_1 = (char)(acc_1 >> 8) ;
	sine_1 = (signed char)((((amp_1>>8) * (int)sineTable[high_1]))>>a_1) ;

	acc_2 = acc_2 + inc_2 ;
	high_2 = (char)(acc_2 >> 8) ;
	sine_2 = (signed char)((((amp_2>>8) * (int)sineTable[high_2]))>>a_2) ;

	acc_3 = acc_3 + inc_3 ;
	high_3 = (char)(acc_3 >> 8) ;
	sine_3 = (signed char)((((amp_3>>8) * (int)sineTable[high_3]))>>a_3) ;
	
	acc_4 = acc_4 + inc_4 ;
	high_4 = (char)(acc_4 >> 8) ;
	sine_4 = (signed char)((((amp_4>>8) * (int)sineTable[high_4]))>>a_4) ;
	
	acc_5 = acc_5 + inc_5 ;
	high_5 = (char)(acc_5 >> 8) ;
	sine_5 = (signed char)((((amp_5>>8) * (int)sineTable[high_5]))>>a_5) ;
	
	acc_6 = acc_6 + inc_6 ;
	high_6 = (char)(acc_6 >> 8) ;
	sine_6 = (signed char)((((amp_6>>8) * (int)sineTable[high_6]))>>a_6) ;
	
	acc_7 = acc_7 + inc_7 ;
	high_7 = (char)(acc_7 >> 8) ;
	sine_7 = (signed char)((((amp_7>>8) * (int)sineTable[high_7]))>>a_7) ;
	
	acc_8 = acc_8 + inc_8 ;
	high_8 = (char)(acc_8 >> 8) ;
	sine_8 = (signed char)((((amp_8>>8) * (int)sineTable[high_8]))>>a_8) ;
	
	acc_9 = acc_9 + inc_9 ;
	high_9 = (char)(acc_9 >> 8) ;
	sine_9 = (signed char)((((amp_9>>8) * (int)sineTable[high_9]))>>a_9) ;
	
	acc_10 = acc_10 + inc_10 ;
	high_10 = (char)(acc_10 >> 8) ;
	sine_10 = (signed char)((((amp_10>>8) * (int)sineTable[high_10]))>>a_10) ;
	
	// output the wavefrom sample
	// scale amplitude to use only high byte and shift into range
	// 0 to 255
	if (freq < FS4)
	begin
		OCR0A = 128 + (sine_1 >> vol) + (sine_2 >> vol) + (sine_3 >> vol) + (sine_4 >> vol) + (sine_5 >> vol) + (sine_6 >> vol) + (sine_7 >> vol) + (sine_8 >> vol) + (sine_9 >> vol) + (sine_10 >> vol) ;
	end
	else
	begin
		OCR0A = 128 + (sine_1 >> (vol-1)) + (sine_2 >> (vol-1)) + (sine_3 >> (vol-1)) + (sine_4 >> (vol-1)) + (sine_5 >> (vol-1)) + (sine_6 >> (vol-1)) + (sine_7 >> (vol-1)) + (sine_8 >> (vol-1)) + (sine_9 >> (vol-1)) + (sine_10 >> (vol-1)) ;
	end
	time++;     //ticks at 8 KHz 
	probe = TCNT1 - probe;
	ok_read = 1;
end 

//**********************************************************       
// Entry point and task scheduler loop
int main(void)
begin  
	initialize();
	
	while (1)
	begin   
		//get the sample  
 		Ain = ADCH;
 		//start another conversion
 		ADCSRA |= (1<<ADSC) ; 

		if (ok_read == 1)
		begin
			 //fprintf(stdout, "Probe is reading %d \r\n", probe);
		end
		
		if (Ain > 90)
		begin
			
			count_val = 0;
			//vol = 4;
			if (Ain > 150)
			begin
				vol = 1;
			end

			if (play_note == 0)
			begin
				pluck = 1;
				play_note = 1;
			end
			// Transition to next note in sequence
			check_fingering();

			// Updated main exp for each voice
			inc_1 = (int)(8.192 * freq) ; 
			inc_2 =  (int)(8.192 * freq * 2) ;
			inc_3 =  (int)(8.192 * freq * 3) ;
			inc_4 = (int)(8.192 * freq * 4) ; 
			inc_5 =  (int)(8.192 * freq * 5) ;
			inc_6 =  (int)(8.192 * freq * 6) ;
			inc_7 = (int)(8.192 * freq * 7) ; 
			inc_8 =  (int)(8.192 * freq * 8) ;
			inc_9 =  (int)(8.192 * freq * 9) ;
			inc_10 =  (int)(8.192 * freq * 10) ;
						
			// Update the amplitudes based on current frequency
			if (freq < 138.0)
			begin
			
				// Set number of terms
				num_harmonics = 10;
				
				// Set corresponding amplitudes
				a_1  = 10;
				a_2  = 10;
				a_3  = 8;
				a_4  = 8;
				a_5  = 11;
				a_6  = 12;
				a_7  = 10;
				a_8  = 10;
				a_9  = 11;
				a_10 = 11;	
			end
			
			else if (freq < 215)
			begin
			
				// Set number of terms
				num_harmonics = 10;
				
				// Set corresponding amplitudes
				a_1  = 8;
				a_2  = 9;
				a_3  = 10;
				a_4  = 10;
				a_5  = 10;
				a_6  = 10;
				a_7  = 9;
				a_8  = 10;
				a_9  = 10;
				a_10 = 9;	
			end
			
			else if (freq < 308)
			begin
			
				// Set number of terms
				num_harmonics = 10;
				
				// Set corresponding amplitudes
				a_1  = 9;
				a_2  = 8;
				a_3  = 10;
				a_4  = 8;
				a_5  = 9;
				a_6  = 9;
				a_7  = 9;
				a_8  = 8;
				a_9  = 9;
				a_10 = 9;
			end
			
			else 
			begin
			
				// Set number of terms
				num_harmonics = 10;
				
				// Set corresponding amplitudes
				a_1  = 10;
				a_2  = 8;
				a_3  = 10;
				a_4  = 11;
				a_5  = 10;
				a_6  = 10;
				a_7  = 11;
				a_8  = 12;
				a_9  = 14;
				a_10 = 14;	
			end
			//results to hyperterm
			fprintf(stdout, "The val is %d\n\r",Ain);
		end
		else
		begin
			play_note = 0;
			//fprintf(stdout, "We are below threshold\r\n");
		end
	end

	
end  


//******************************* 
// check_fingering: Adjust frequency based on current button fingering
void check_fingering()
begin

	// Check each button to see if it is pressed
	key1_on  = 0 < (~PINA & 0x08);
	key2_on  = 0 < (~PINC & 0x08);
	key3_on  = 0 < (~PINC & 0x01);
	key4_on  = 0 < (~PINC & 0x10);
	key5_on  = 0 < (~PINC & 0x40);
	key6_on  = 0 < (~PINC & 0x80);
	key7_on  = 0 < (~PIND & 0x40);
	key8_on  = 0 < (~PIND & 0x01);
	key9_on  = 0 < (~PINC & 0x04);
	key10_on = 0 < (~PIND & 0x04);
	key11_on = 0 < (~PINC & 0x02);
	key12_on = 0 < (~PINC & 0x20);
	key13_on = 0 < (~PIND & 0x20);
	key14_on = 0 < (~PIND & 0x10);
	key15_on = 0 < (~PIND & 0x08);
	key16_on = 0 < (~PINA & 0x10);
	key17_on = 0 < (~PINA & 0x40);
	key18_on = 0 < (~PINA & 0x20);
	key19_on = 0 < (~PINA & 0x80);
	
	// Encode key fingerings into three binary numbers
	portd_keys = 128*key1_on + 64*key2_on + 32*key3_on + 16*key4_on + 8*key5_on + 4*key6_on + 2*key7_on + key8_on;
	portc_keys = 128*key9_on + 64*key10_on + 32*key11_on + 16*key12_on + 8*key13_on + 4*key14_on + 2*key15_on + key16_on;
	porta_keys = 4*key17_on + 2*key18_on + key19_on;
	
	// B flat = A#2
	if (portd_keys == 0b11111101 && portc_keys == 0b00010000 && porta_keys == 0b000  )
	begin
		freq = AS2;
	end
	
	// B2
	else if (portd_keys == 0b11111101 && portc_keys == 0b01000000 && porta_keys == 0b000 )
	begin
		freq = B2;
	end
	
	// C3
	else if (portd_keys == 0b11111101 && portc_keys == 0b00000000 && porta_keys == 0b000 )
	begin
		freq = C3;
	end
	
	// C#3 or D flat
	else if (portd_keys == 0b11111101 && portc_keys == 0b00100000 && porta_keys == 0b000)
	begin
		freq = CS3;
	end
	
	// D3
	else if (portd_keys == 0b11111100 && portc_keys == 0b00000000 && porta_keys == 0b000 )
	begin
		freq = D3;
	end
	
	// D#3 or E flat 
	else if (portd_keys == 0b11111110 && portc_keys == 0b00000000 && porta_keys == 0b000)
	begin
		freq = DS3;
	end
	
	// E3
	else if (portd_keys == 0b11111000 && portc_keys == 0b00000000 && porta_keys == 0b000)
	begin
		freq = E3;
	end
	
	// F3
	else if (portd_keys == 0b11110000 && portc_keys == 0b00000000 && porta_keys == 0b000)
	begin
		freq = F3;
	end
	
	// F#3 or G flat 3
	else if (portd_keys == 0b11101000 && portc_keys == 0b00000000 && porta_keys == 0b000)	  
	begin
		freq = FS3;
	end
	
	// G3
	else if (portd_keys == 0b11100000 && portc_keys == 0b00000000 && porta_keys == 0b000)
	begin
		freq = G3;
	end
	
	// G#3 or A flat 3
	else if (portd_keys == 0b11100000 && portc_keys == 0b10000000 && porta_keys == 0b000)
	begin
		freq = GS3;
	end
	
	// A3
	else if (portd_keys == 0b11000000 && portc_keys == 0b00000000 && porta_keys == 0b000)
	begin
		freq = A3;
	end
	
	// A#3 or B flat - 3 fingerings
	else if ( (portd_keys == 0b11000000 && portc_keys == 0b00000010 && porta_keys == 0b000) ||
			  (portd_keys == 0b10010000 && portc_keys == 0b00000000 && porta_keys == 0b000) )
	begin
		freq = AS3;
	end
	
	// B3
	else if (portd_keys == 0b10000000 && portc_keys == 0b00000000 && porta_keys == 0b000)
	begin
		freq = B3;
	end
	
	// C4 - 2 fingerings
	else if ( (portd_keys == 0b01000000 && portc_keys == 0b00000000 && porta_keys == 0b000) ||
			  (portd_keys == 0b10000000 && portc_keys == 0b00000100 && porta_keys == 0b000) )
	begin
		freq = C4;
	end
	
	// C#4 or D flat
	else if (portd_keys == 0b00000000 && portc_keys == 0b00000000 && porta_keys == 0b000)
	begin
		freq = CS4;
	end
	
	// D4
	else if (portd_keys == 0b11111100 && portc_keys == 0b00000001 && porta_keys == 0b000)
	begin
		freq = D4;
	end
	
	// D#4 or E flat 4
	else if (portd_keys == 0b11111110 && portc_keys == 0b00000001 && porta_keys == 0b000)
	begin
		freq = DS4;
	end
	
	// E4
	else if (portd_keys == 0b11111000 && portc_keys == 0b00000001 && porta_keys == 0b000)
	begin
		freq = E4;
	end
	
	// F4
	else if (portd_keys == 0b11110000 && portc_keys == 0b00000001 && porta_keys == 0b000)
	begin
		freq = F4;
	end
	
	// F#4 or G flat 4
	else if (portd_keys == 0b11101000 && portc_keys == 0b00000001 && porta_keys == 0b000)
	begin
		freq = FS4;
	end
	
	// G4
	else if (portd_keys == 0b11100000 && portc_keys == 0b00000001 && porta_keys == 0b000)
	begin
		freq = G4;
	end
	
	// GS4 or A flat 4
	else if (portd_keys == 0b11100000 && portc_keys == 0b10000001 && porta_keys == 0b000)
	begin
		freq = GS4;
	end
	
	// A4
	else if (portd_keys == 0b11000000 && portc_keys == 0b00000001 && porta_keys == 0b000)
	begin
		freq = A4;
	end
	
	// A#4 or B flat 4 - 3 fingerings
	else if ( (portd_keys == 0b11000000 && portc_keys == 0b00000011 && porta_keys == 0b000) ||
			  (portd_keys == 0b10010000 && portc_keys == 0b00000001 && porta_keys == 0b000) )
	begin
		freq = AS4;
	end
	
	// B4
	else if (portd_keys == 0b10000000 && portc_keys == 0b00000001 && porta_keys == 0b000)
	begin
		freq = B4;
	end
	
	// C5 - 2 fingerings
	else if ( (portd_keys == 0b01000000 && portc_keys == 0b00000001 && porta_keys == 0b000) ||
			  (portd_keys == 0b10000000 && portc_keys == 0b00000101 && porta_keys == 0b000) )
	begin
		freq = C5;
	end
	
	// C#5 or D flat 5
	else if (portd_keys == 0b00000000 && portc_keys == 0b00000001 && porta_keys == 0b000)
	begin
		freq = CS5;
	end
	
	// D5
	else if (portd_keys == 0b00000000 && portc_keys == 0b00000001 && porta_keys == 0b010)
	begin
		freq = D5;
	end
	
	// DS5 or E flat 5
	else if (portd_keys == 0b00000000 && portc_keys == 0b00000001 && porta_keys == 0b110)
	begin
		freq = DS5;
	end
	
	// E5
	else if (portd_keys == 0b00000000 && portc_keys == 0b00001001 && porta_keys == 0b110)
	begin
		freq = E5;
	end
	
	// F5 
	else if (portd_keys == 0b00000000 && portc_keys == 0b00001001 && porta_keys == 0b111) 
	begin
		freq = F5;
	end

	// DEFAULT NOTE
	else
	begin
		freq = CS5;
	end
	
	/* BUTTON TEST CODE: TELLS YOU WHICH KEY IS PRESSED */
	if      (key1_on)  { fprintf(stdout, "Key 1  is pressed \r\n"); }
	else if (key2_on)  { fprintf(stdout, "Key 2  is pressed \r\n"); }
	else if (key3_on)  { fprintf(stdout, "Key 3  is pressed \r\n"); }
	else if (key4_on)  { fprintf(stdout, "Key 4  is pressed \r\n"); }
	else if (key5_on)  { fprintf(stdout, "Key 5  is pressed \r\n"); }
	else if (key6_on)  { fprintf(stdout, "Key 6  is pressed \r\n"); }
	else if (key7_on)  { fprintf(stdout, "Key 7  is pressed \r\n"); }
	else if (key8_on)  { fprintf(stdout, "Key 8  is pressed \r\n"); }
	else if (key9_on)  { fprintf(stdout, "Key 9  is pressed \r\n"); }
	else if (key10_on) { fprintf(stdout, "Key 10 is pressed \r\n"); }
	else if (key11_on) { fprintf(stdout, "Key 11 is pressed \r\n"); }
	else if (key12_on) { fprintf(stdout, "Key 12 is pressed \r\n"); }
	else if (key13_on) { fprintf(stdout, "Key 13 is pressed \r\n"); }
	else if (key14_on) { fprintf(stdout, "Key 14 is pressed \r\n"); }
	else if (key15_on) { fprintf(stdout, "Key 15 is pressed \r\n"); }
	else if (key16_on) { fprintf(stdout, "Key 16 is pressed \r\n"); }
	else if (key17_on) { fprintf(stdout, "Key 17 is pressed \r\n"); }
	else if (key18_on) { fprintf(stdout, "Key 18 is pressed \r\n"); }
	else if (key19_on) { fprintf(stdout, "Key 19 is pressed \r\n"); }

end


  
//********************************************************** 
//Set it all up
void initialize(void)
begin
  	
	// Set initial note
	freq = CS5;

	uart_init();
  	stdout = stdin = stderr = &uart_str;
  	fprintf(stdout,"Starting...\n\r");



	// Make B.3 an output
	DDRB = (1<<PINB3);

	// Make Port D an input
	DDRD = 0x00;

	// Make Port A an input
	PORTA = 0x00;
	DDRA = 0x00;

	// Make Port C an input
	DDRC = 0x00;
	

	// init the sine table
	for (i=0; i<256; i++)
	begin
		sineTable[i] = (char)(127.0 * sin(6.283*((float)i)/256.0));
	end

	///////////////////////////////////////////////////
    // Sound parameters
    ///////////////////////////////////////////////////
    //  frequency 2^16/8000*freq = 8.192*freq
	// rise and decay SHIFT factors  -- bigger is slower -- max value is 8
	// amplitude SHIFT factor  -- bigger is lower amp -- min value is 7
	
   	inc_1 = (int)(8.192 * freq) ;     
    fall_1 = 2 ;
    rise_1 = 0 ;    
    a_1 = 8 ;
    
    inc_2 =  (int)(8.192 * freq * 2) ;
	fall_2 = 2 ;
	rise_2 = 0 ;
	a_2 = 8 ;
	
	inc_3 =  (int)(8.192 * freq * 3) ;
	fall_3 = 2 ;
	rise_3 = 0 ;
	a_3 = 10 ;
	
	inc_4 =  (int)(8.192 * freq * 4) ;
	fall_4 = 2 ;
	rise_4 = 0 ;
	a_4 = 10 ;
	
	inc_5 =  (int)(8.192 * freq * 5) ;
	fall_5 = 2 ;
	rise_5 = 0 ;
	a_5 = 10 ;
	
	inc_6 =  (int)(8.192 * freq * 6) ;
	fall_6 = 2 ;
	rise_6 = 0 ;
	a_6 = 10 ;
	
	inc_7 =  (int)(8.192 * freq * 7) ;
	fall_7 = 2 ;
	rise_7 = 0 ;
	a_7 = 10 ;
	
	inc_8 =  (int)(8.192 * freq * 8) ;
	fall_8 = 2 ;
	rise_8 = 0 ;
	a_8 = 10 ;
	
	inc_9 =  (int)(8.192 * freq * 9) ;
	fall_9 = 2 ;
	rise_9 = 0 ;
	a_9 = 10 ;
	
	inc_10 =  (int)(8.192 * freq * 10) ;
	fall_10 = 2 ;
	rise_10 = 0 ;
	a_10 = 10 ;
	
	// init the time counters
	time = 0;
	time1=t1;   
	time2=t2;	
	time3=t3;
	
	// timer 0 runs at full rate
	TCCR0B = 1 ;  
	//turn off timer 0 overflow ISR
	TIMSK0 = 0 ;
	// turn on PWM
	// turn on fast PWM and OC0A output
	// at full clock rate, toggle OC0A (pin B3) 
	// 16 microsec per PWM cycle sample time
	TCCR0A = (1<<COM0A0) | (1<<COM0A1) | (1<<WGM00) | (1<<WGM01) ; 
	OCR0A = 128 ; // set PWM to half full scale
	
	// timer 1 ticks at 8000 Hz or 125 microsecs period=2000 ticks
	OCR1A = 1999 ; // 2000 ticks
	TIMSK1 = (1<<OCIE1A) ;
	TCCR1B = 0x09; 	    //full speed; clear-on-match
  	TCCR1A = 0x00;      //turn off pwm and oc lines
		
	/*
	// Timer 2 used to update LCD and schedule tasks
	TIMSK2 = 2;         // turn on timer 2 cmp match ISR
	OCR2A = 250;		// set the compare reg to 250 time ticks
	TCCR2A = 0x02;		// turn on clear-on-match
	TCCR2B = 0x04;		// clock prescalar to 64
	

	EIMSK = 1<<INT0 ; // turn on int0
	EICRA = 3 ;       // rising edge
	// turn on timer 2 to be read in int0 ISR
	TCCR2B = 7 ; // divide by 1024
	// turn on timer 2 overflow ISR for double precision time
	TIMSK2 = 1 ;
	*/
	
	//init the A to D converter 
   //channel zero/ left adj /EXTERNAL Aref
   ADMUX = (1<<ADLAR);
   ADMUX |= 0x40;  
    
   //enable ADC and set prescaler to 1/128*16MHz=125,000
   //and clear interupt enable
   //and start a conversion
   ADCSRA = ((1<<ADEN) | (1<<ADSC)) + 7 ; 

   play_note = 0;
   count_val = 0;

	//crank up the ISRs
	sei();
end  

   
